]> git.sesse.net Git - nageru/blobdiff - mixer.cpp
Start pulling video orchestration logic into VideoEncoder.
[nageru] / mixer.cpp
index efd4952b7e565f379128673e11f2a7089d46c644..dc9eea63fc0473f99d9260bffb8c99bb10aa485d 100644 (file)
--- a/mixer.cpp
+++ b/mixer.cpp
@@ -33,7 +33,7 @@
 #include "decklink_capture.h"
 #include "defs.h"
 #include "flags.h"
-#include "quicksync_encoder.h"
+#include "video_encoder.h"
 #include "pbo_frame_allocator.h"
 #include "ref_counted_gl_sync.h"
 #include "timebase.h"
@@ -92,23 +92,6 @@ void insert_new_frame(RefCountedFrame frame, unsigned field_num, bool interlaced
        }
 }
 
-string generate_local_dump_filename(int frame)
-{
-       time_t now = time(NULL);
-       tm now_tm;
-       localtime_r(&now, &now_tm);
-
-       char timestamp[256];
-       strftime(timestamp, sizeof(timestamp), "%F-%T%z", &now_tm);
-
-       // Use the frame number to disambiguate between two cuts starting
-       // on the same second.
-       char filename[256];
-       snprintf(filename, sizeof(filename), "%s%s-f%02d%s",
-               LOCAL_DUMP_PREFIX, timestamp, frame % 100, LOCAL_DUMP_SUFFIX);
-       return filename;
-}
-
 }  // namespace
 
 void QueueLengthPolicy::update_policy(int queue_length)
@@ -174,10 +157,9 @@ Mixer::Mixer(const QSurfaceFormat &format, unsigned num_cards)
        display_chain->set_dither_bits(0);  // Don't bother.
        display_chain->finalize();
 
-       quicksync_encoder.reset(new QuickSyncEncoder(h264_encoder_surface, global_flags.va_display, WIDTH, HEIGHT, &httpd));
-       quicksync_encoder->open_output_file(generate_local_dump_filename(/*frame=*/0).c_str());
+       video_encoder.reset(new VideoEncoder(h264_encoder_surface, global_flags.va_display, WIDTH, HEIGHT, &httpd));
 
// Start listening for clients only once H264Encoder has written its header, if any.
        httpd.start(9095);
 
        // First try initializing the PCI devices, then USB, until we have the desired number of cards.
@@ -292,7 +274,7 @@ Mixer::~Mixer()
                cards[card_index].capture->stop_dequeue_thread();
        }
 
-       quicksync_encoder.reset(nullptr);
+       video_encoder.reset(nullptr);
 }
 
 void Mixer::configure_card(unsigned card_index, const QSurfaceFormat &format, CaptureInterface *capture)
@@ -692,12 +674,7 @@ void Mixer::thread_func()
                }
 
                if (should_cut.exchange(false)) {  // Test and clear.
-                       string filename = generate_local_dump_filename(frame);
-                       printf("Starting new recording: %s\n", filename.c_str());
-                       quicksync_encoder->close_output_file();
-                       quicksync_encoder->shutdown();
-                       quicksync_encoder.reset(new QuickSyncEncoder(h264_encoder_surface, global_flags.va_display, WIDTH, HEIGHT, &httpd));
-                       quicksync_encoder->open_output_file(filename.c_str());
+                       video_encoder->do_cut(frame);
                }
 
 #if 0
@@ -784,7 +761,7 @@ void Mixer::render_one_frame(int64_t duration)
        //theme_main_chain.chain->enable_phase_timing(true);
 
        GLuint y_tex, cbcr_tex;
-       bool got_frame = quicksync_encoder->begin_frame(&y_tex, &cbcr_tex);
+       bool got_frame = video_encoder->begin_frame(&y_tex, &cbcr_tex);
        assert(got_frame);
 
        // Render main chain.
@@ -806,7 +783,7 @@ void Mixer::render_one_frame(int64_t duration)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
 
        const int64_t av_delay = TIMEBASE / 10;  // Corresponds to the fixed delay in resampling_queue.h. TODO: Make less hard-coded.
-       RefCountedGLsync fence = quicksync_encoder->end_frame(pts_int + av_delay, duration, theme_main_chain.input_frames);
+       RefCountedGLsync fence = video_encoder->end_frame(pts_int + av_delay, duration, theme_main_chain.input_frames);
 
        // The live frame just shows the RGBA texture we just rendered.
        // It owns rgba_tex now.
@@ -1027,7 +1004,7 @@ void Mixer::process_audio_one_frame(int64_t frame_pts_int, int num_samples)
        }
 
        // And finally add them to the output.
-       quicksync_encoder->add_audio(frame_pts_int, move(samples_out));
+       video_encoder->add_audio(frame_pts_int, move(samples_out));
 }
 
 void Mixer::subsample_chroma(GLuint src_tex, GLuint dst_tex)
@@ -174,10 +157,9 @@ Mixer::Mixer(const QSurfaceFormat &format, unsigned num_cards)
 
-       // Start listening for clients only once H264Encoder has written its header, if any.
+       // Start listening for clients only once VideoEncoder has written its header, if any.
        httpd.start(9095);
 